package org.diagbot.nlp.relation.analyze;

import org.algorithm.core.cnn.AlgorithmCNNExecutor;
import org.algorithm.core.cnn.entity.Lemma;
import org.algorithm.core.cnn.entity.Triad;
import org.algorithm.core.cnn.model.impl.RelationExtractionModelImpl;
import org.diagbot.nlp.feature.FeatureType;
import org.diagbot.nlp.participle.word.Lexeme;
import org.diagbot.nlp.participle.word.LexemePath;
import org.diagbot.nlp.relation.extract.*;
import org.diagbot.nlp.relation.module.*;
import org.diagbot.nlp.relation.module.cell.*;
import org.diagbot.nlp.relation.util.LemmaUtil;
import org.diagbot.nlp.relation.util.OutputInfo;
import org.diagbot.nlp.util.Constants;
import org.omg.Messaging.SYNC_WITH_TRANSPORT;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import static org.diagbot.nlp.participle.ParticipleUtil.participle;

/**
 * @ClassName org.diagbot.nlp.relation.StructureAnalyze
 * @Description 结构分析入口
 * @Author Mark Huang
 * @Date 2019/4/30 9:07
 * @Version 1.0
 **/
public class StructureAnalyze {

    private List<OutputInfo> outputInfos = new ArrayList<>();
    private static boolean updated = false;
    private static Object current = null;
    private static Object current_treat = null;
    private static BodyPart bodyPart = null;
    private static String num = "";
    private static List<String> diag_con = Arrays.asList("门诊","诊断");
    private static List<String> past_con = Arrays.asList("既往");

    /**
     * 分词，提取词性，封装
     */
    public List<OutputInfo> extract(String content) {

        Lexeme lexeme = null;
        String timestamp = "";
        String[] tokens = content.split("，|。|；");

        try {
//            outputInfos.add(initOutputInfo());
            for (String token:tokens) {
                updated = false;
                outputInfos.add(initOutputInfo());
                OutputInfo outputInfo = outputInfos.get(outputInfos.size()-1);
                System.out.println(token);
                LexemePath<Lexeme> lexemePath = participle(token, false);
                for (int i = 0; i < lexemePath.size(); i++) {
                    lexeme = lexemePath.get(i);
                    updateLexeme(lexeme);
                    updateInfo(outputInfo, lexeme);
                }

//                Symptom symptom = outputInfos.get(outputInfos.size() - 1).getSymptoms().get(0);

                if (updated == false ||
                        (getPropsize(outputInfo) == 1 && outputInfo.getSymptoms().get(0).getSymptomName().trim().length() == 0)){
                    outputInfos.remove(outputInfos.size() - 1);
                }

                bodyPart = null;
            }
        }
        catch (IOException ioe) {
            ioe.printStackTrace();
        }
        finally {
            current = null;
            return outputInfos;
        }
    }

    public void updateLexeme(Lexeme lexeme) {
        String prop = lexeme.getProperty();
        String[] lexprop = lexeme.getProperty().split(",");

        if (lexprop.length > 1) {
            prop = lexprop[0];
            for (int i=0;i<lexprop.length;i++) {
                if (current instanceof Treat && Arrays.asList(lexprop).indexOf(Constants.word_property_diagnose)>=0) {
                    prop = Constants.word_property_diagnose;
                }
                else if (Integer.parseInt(lexprop[i]) < Integer.parseInt(prop)) {
                    prop = lexprop[i];
                }
            }
        }
        lexeme.setProperty(prop);
    }

    public void updateInfo(OutputInfo outputInfo, Lexeme lexeme) {
        String con = "伴";
        Symptom symptom = outputInfo.getSymptoms().get(outputInfo.getSymptoms().size() - 1);;
        Lis lis = null;
        Pacs pacs = null;
        Vital vital = null;
        Treat treat = null;
        Past past = null;
        PD pd1 = null;

        try {

            switch (lexeme.getProperty()) {
                case Constants.word_property_prop:

                case Constants.word_property_degree:
                case Constants.word_property_degree_quan:

                case Constants.word_property_cause:
                case Constants.word_property_symptom:
                case Constants.word_property_vital_idx:
                case Constants.word_property_vital_result:
                    current = updateClinicalInfo(outputInfo, lexeme.getText(), lexeme.getProperty());
                    updated = true;
                    if (bodyPart != null) {
                        switch (lexeme.getProperty()) {
                            case Constants.word_property_symptom:
                                if (current instanceof Symptom) {
                                    ((Symptom)current).setBodyPart(bodyPart);
                                    bodyPart = null;
                                }
                                break;
                            case Constants.word_property_vital_idx:
                            case Constants.word_property_vital_result:
                                if (current instanceof Vital) {
                                    ((Vital)current).setBodyPart(bodyPart);
                                    bodyPart = null;
                                }
                                break;
                            case Constants.word_property_PACS:
                                if (current instanceof Pacs) {
                                    ((Pacs)current).setBodypart(bodyPart.getPartBodyName());
                                    bodyPart = null;
                                }
                                break;
                        }
                    }
                    break;
                case Constants.word_property_vital_value:
                    if (current instanceof Vital) {
                        vital = (Vital) current;
                        if (vital.getPd() == null) {
                            vital.setPd(new PD());
                        }
                        pd1 = vital.getPd();
                        pd1.setValue(lexeme.getText());
                        pd1.setUnit("");
                        updated = true;
                    }
                    break;
                case Constants.word_property_LIS:
                case Constants.word_property_LIS_Detail:
                    lis = new Lis();
                    lis.setLisName(lexeme.getText());
                    outputInfo.getLises().add(lis);
                    current = lis;
                    updated = true;
                    break;
                case Constants.word_property_LIS_Result:
                    if (current instanceof Lis) {
                        if (((Lis)current).getPd() == null) {
                            ((Lis)current).setPd(new PD());
                            pd1 = ((Lis)current).getPd();
                        }
                        pd1.setValue(lexeme.getText());
                        pd1.setUnit("");
                        updated = true;
                    }
                    break;
                case Constants.word_property_PACS:
                    pacs = new Pacs();
                    pacs.setPacsName(lexeme.getText());
                    outputInfo.getPacses().add(pacs);
                    current = pacs;
                    updated = true;
                    break;
                case Constants.word_property_PACS_Result:
                    if (current instanceof Pacs) {
                        if (((Pacs) current).getValue() == null) {
                            ((Pacs) current).setValue("");
                        }
                        ((Pacs) current).setValue(((Pacs) current).getValue() + lexeme.getText() + ", ");
                        updated = true;
                    }
                    break;
                case Constants.word_property_bodypart:
                    if (bodyPart == null) {
                        bodyPart = new BodyPart();
                        bodyPart.setPartBodyName(lexeme.getText());
                        updated = true;
                    }
                    break;
                case Constants.word_property_direction:
                    if (current instanceof Pacs) {
                        if (((Pacs)current).getBodypart()==null) {
                            ((Pacs)current).setBodypart("");
                        }
                        ((Pacs) current).setBodypart(((Pacs)current).getBodypart()+lexeme.getText() + ", ");
                        updated = true;
                    }
                    else if (current instanceof Symptom) {
                        PD pd = new PD();
                        pd.setValue(lexeme.getText());
                        pd.setUnit("");
                        if (((Symptom)current).getPds() == null) {
                            ((Symptom)current).setPds(new ArrayList<>());
                        }
                        ((Symptom)current).getPds().add(pd);
                        updated = true;
                    }
                    break;
                case Constants.word_property_neg:
                    Negative negative = new Negative();
                    negative.setNegaName(lexeme.getText());
                    symptom.setNegative(negative);
                    updated = true;
                    break;
                case Constants.word_property_timestamp:
                    PD pd = new PD();
                    pd.setValue(lexeme.getText());
                    pd.setUnit("");
                    List<PD> pds = new ArrayList<>();
                    pds.add(pd);
                    if (current instanceof Symptom) {
                        symptom = (Symptom)current;
                    }
                    if (symptom != null) {
                        symptom.setTimestamp(pds);
                        updated = true;
                        current = symptom;
                    }
                    break;
                case Constants.word_property_freq:
                case Constants.word_property_number:
                    num = lexeme.getText();

                    updated = true;

                    break;
                case Constants.word_property_unit:
                    if (num.length() > 0) {
                        if (current instanceof Symptom) {
                            symptom = (Symptom) current;
                            if (symptom.getPds() == null) {
                                List<PD> pds1 = new ArrayList<>();
                                pds1.add(new PD());
                                symptom.setPds(pds1);
                            }
                            pd1 = symptom.getPds().get(0);
                        } else if (current instanceof Vital) {
                            vital = (Vital) current;
                            if (vital.getPd() == null) {
                                vital.setPd(new PD());
                            }
                            pd1 = vital.getPd();
                        } else if (current instanceof Treat) {
                            treat = (Treat)current;
                            if (treat.getPds() == null) {
                                List<PD> pds1 = new ArrayList<>();
                                pds1.add(new PD());
                                treat.setPds(pds1);
                            }
                            pd1 = treat.getPds().get(0);
                        } else if (current instanceof Lis) {
                            lis = (Lis)current;
                            if (lis.getPd() == null) {
                                lis.setPd(new PD());
                            }
                            pd1 = lis.getPd();
                        } else if (current instanceof Past) {
                            past = (Past)current;
                            if (past.getPds() == null) {
                                List<PD> pds1 = new ArrayList<>();
                                pds1.add(new PD());
                                past.setPds(pds1);
                            }
                            pd1 = past.getPds().get(0);
                        }
                    }
                    pd1.setValue(num);
                    pd1.setUnit(lexeme.getText());
                    updated = true;
                    num = "";
                    break;
                case Constants.word_property_time:
                    if (num.length() > 0) {
                        if (symptom.getTimestamp() == null) {
                            List<PD> pds1 = new ArrayList<>();
                            pds1.add(new PD());
                            symptom.setTimestamp(pds1);
                        }
                        pd1 = symptom.getTimestamp().get(0);
                        pd1.setValue(num);
                        pd1.setUnit(lexeme.getText());
                        updated = true;
                        num = "";
                        current = symptom;
                    }
                    break;
                case Constants.word_property_diagnose:
                case Constants.word_property_med:
                case Constants.word_property_treat:
                    current = updateClinicalInfo(outputInfo, lexeme.getText(), lexeme.getProperty());
                    updated = true;
                    break;
                case Constants.word_property_other:
                    if (diag_con.indexOf(lexeme.getText()) >= 0) {
                        current = new Treat();
                    }
                    else if (past_con.indexOf(lexeme.getText()) >= 0) {
                        current = new Past();
                    }
                    break;
            }
        }
        catch (Exception ex) {
            ex.printStackTrace();
        }
    }

    public OutputInfo initOutputInfo() {
        OutputInfo outputInfo = new OutputInfo();

        outputInfo.setSymptoms(new ArrayList<>());
        outputInfo.getSymptoms().add(new Symptom());
        outputInfo.getSymptoms().get(outputInfo.getSymptoms().size()-1).setSymptomName("");

        outputInfo.setVitals(new ArrayList<>());

        outputInfo.setPasts(new ArrayList<>());

        outputInfo.setLises(new ArrayList<>());

        outputInfo.setPacses(new ArrayList<>());

        outputInfo.setTreats(new ArrayList<>());

        return outputInfo;
    }

    public int getPropsize(OutputInfo outputInfo) {
        int size = outputInfo.getSymptoms().size() + outputInfo.getVitals().size() +
                outputInfo.getPasts().size() + outputInfo.getLises().size() +
                outputInfo.getPacses().size() + outputInfo.getTreats().size();

        return size;
    }


    public Object updateClinicalInfo(OutputInfo outputInfo, String name, String type) {

        Object obj = null;
        Symptom symptom = null;
        Treat treat = null;
        Past past = null;

        switch (type) {
            case Constants.word_property_symptom:
                symptom = outputInfo.getSymptoms().get(outputInfo.getSymptoms().size() - 1);

                if (symptom.getSymptomName() == null || symptom.getSymptomName().trim().length() == 0) {
                    symptom.setSymptomName(name);
                }
                else {
                    symptom.setSymptomName(symptom.getSymptomName() + ", " + name);
                }

                obj = symptom;
                break;
            case Constants.word_property_vital_idx:
            case Constants.word_property_vital_result:
                if (outputInfo.getVitals().size() == 0) {
                    outputInfo.getVitals().add(new Vital());
                }

                Vital vital = outputInfo.getVitals().get(outputInfo.getVitals().size()-1);

                if (vital.getVitalName() == null || vital.getVitalName().trim().length() == 0) {
                    vital.setVitalName(name);
                }
                else {
                    vital.setVitalName(vital.getVitalName() + "," + name);
                }

                obj = vital;
                break;
            case Constants.word_property_diagnose:
                if (current instanceof Past) {
                    past = (Past)current;

                    if (past.getDiagnose() == null || past.getDiagnose().trim().length() == 0) {
                        past.setDiagnose(name);
                    } else {
                        past.setDiagnose(past.getDiagnose() + "," + name);
                    }
                    outputInfo.getPasts().add(past);
                    obj = past;
                    current_treat = past;
                }
                else {
                    if (current instanceof Treat) {
                        treat = (Treat) current;
                    } else if (outputInfo.getTreats().size() == 0) {
                        outputInfo.getTreats().add(new Treat());
                        treat = outputInfo.getTreats().get(outputInfo.getTreats().size() - 1);
                    }

                    if (treat.getDiagnose() == null || treat.getDiagnose().trim().length() == 0) {
                        treat.setDiagnose(name);
                    } else {
                        treat.setDiagnose(treat.getDiagnose() + "," + name);
                    }
                    obj = treat;
                    current_treat = treat;
                }
                break;
            case Constants.word_property_cause:
                Cause cause = new Cause();
                cause.setCauseName(name);

                if (current instanceof Symptom) {
                    symptom = (Symptom)current;
                }
                else {
                    symptom = outputInfo.getSymptoms().get(outputInfo.getSymptoms().size() - 1);
                }

                if (symptom.getCause() != null) {
                    if (!outputInfo.getSymptoms().get(outputInfo.getSymptoms().size() - 1).getSymptomName().equals(symptom.getSymptomName())) {
                        outputInfo.getSymptoms().get(outputInfo.getSymptoms().size() - 1).setSymptomName(symptom.getSymptomName());
                        symptom = outputInfo.getSymptoms().get(outputInfo.getSymptoms().size() - 1);
                    }
                }
                symptom.setCause(cause);

                obj = symptom;

                break;
            case Constants.word_property_degree:
            case Constants.word_property_degree_quan:
                Degree degree = new Degree();
                degree.setDegreeName(name);

                if (current instanceof Symptom) {
                    symptom = (Symptom)current;
                }
                else {
                    symptom = outputInfo.getSymptoms().get(outputInfo.getSymptoms().size() - 1);
                }

                if (symptom.getDegree() != null) {
                    if (!outputInfo.getSymptoms().get(outputInfo.getSymptoms().size() - 1).getSymptomName().equals(symptom.getSymptomName())) {
                        outputInfo.getSymptoms().get(outputInfo.getSymptoms().size() - 1).setSymptomName(symptom.getSymptomName());
                        symptom = outputInfo.getSymptoms().get(outputInfo.getSymptoms().size() - 1);
                    }
                }
                symptom.setDegree(degree);

                obj = symptom;
                break;
            case Constants.word_property_prop:
                Property property = new Property();
                property.setPropertyName(name);

                if (current instanceof Symptom) {
                    symptom = (Symptom)current;
                }
                else {
                    symptom = outputInfo.getSymptoms().get(outputInfo.getSymptoms().size() - 1);
                }

                if (symptom.getProperty() != null) {
                    if (!outputInfo.getSymptoms().get(outputInfo.getSymptoms().size() - 1).getSymptomName().equals(symptom.getSymptomName())) {
                        outputInfo.getSymptoms().get(outputInfo.getSymptoms().size() - 1).setSymptomName(symptom.getSymptomName());
                        symptom = outputInfo.getSymptoms().get(outputInfo.getSymptoms().size() - 1);
                    }
                }
                symptom.setProperty(property);
                obj = symptom;
                break;
            case Constants.word_property_treat:
                if (current instanceof Treat) {
                    treat = (Treat)current;
                    if (treat.getValue() == null || treat.getValue().trim().length() == 0) {
                        treat.setValue(name);
                    }
                    else {
                        treat.setValue(treat.getValue() + "," + name);
                    }
                    current_treat = treat;
                    obj = treat;
                }
                else if (current_treat instanceof Past) {
                    past = (Past)current_treat;
                    if (past.getValue() == null || past.getValue().trim().length() == 0) {
                        past.setValue(name);
                    }
                    else {
                        past.setValue(past.getValue() + "," + name);
                    }

                    obj = past;
                }
                break;
            case Constants.word_property_med:
                if (current_treat instanceof Past) {
                    past = (Past)current_treat;

                    if (past.getTreatName() == null || past.getTreatName().trim().length() == 0) {
                        past.setTreatName(name);
                    } else {
                        past.setTreatName(past.getTreatName() + "," + name);
                    }

                    obj = past;
                }
                else {
                    if (!(current instanceof Treat)) {
                        if (current_treat != null && current_treat instanceof Treat) {
                            current = current_treat;
                        } else {
                            current = updateClinicalInfo(outputInfo, "", Constants.word_property_diagnose);
                        }
                    }
                    treat = (Treat) current;

                    if (treat.getTreatName() == null || treat.getTreatName().trim().length() == 0) {
                        treat.setTreatName(name);
                    } else {
                        treat.setTreatName(treat.getTreatName() + "," + name);
                    }
                    obj = treat;
                }
                break;
        }

        return obj;
    }

}
